unit DesignBtn;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;
const
  NoDrag=0;
  ResLeft=1;
  ResRight=2;
  ResTop=3;
  ResBottom=4;
  ResLeftTop=5;
  ResRightBottom=6;
  ResRightTop=7;
  ResLeftBottom=8;
  DragBtn=9;

type
  StateType=(stDesign,stRun);

  TDesignBtn=class(TButton)
  private
    FixX,FixY:Integer;
    Selected:Boolean;
    Dragging:Byte;
    FOnDesignDblClick: TNotifyEvent;
    FBtnState:StateType;
    procedure WriteBtnState(FState:StateType);
  protected
     procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
     procedure MouseDown(Button: TMouseButton;
         Shift: TShiftState; X, Y: Integer); override;
     procedure MouseMove(Shift: TShiftState; X,
         Y: Integer);override;
     procedure MouseUp(Button: TMouseButton;
         Shift: TShiftState; X, Y: Integer);override;
     procedure DoEnter; override;
     procedure DoExit; override;
     procedure DesignDblClick(var Message: TWMMouse); message WM_LBUTTONDBLCLK;
  public
     constructor Create(AOwner: TComponent); override;
     procedure Click; override;
  published
     property BtnState:StateType read FBtnState write WriteBtnState;
     property OnDesignDblClick: TNotifyEvent read FOnDesignDblClick write FOnDesignDblClick;
   end;

procedure Register;

implementation

constructor TDesignBtn.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   Parent:=TWinControl(AOwner);
   Selected:=False;
   FBtnState:=stRun;
   Caption:=Name;
end;

procedure TDesignBtn.WriteBtnState(FState:StateType);
begin
   if FState=stRun then Selected:=False;
   FBtnState:=FState;
   Cursor:=crDefault;
end;

procedure TDesignBtn.Click;
begin
  if FBtnState= stDesign then InvalidateRect(Handle,nil,True);
  inherited Click;
end;

procedure TDesignBtn.DesignDblClick;
begin
  if (FBtnState=stDesign)and(Assigned(FOnDesignDblClick)) then FOnDesignDblClick(Self);
end;

procedure DrawSquare(Btn:TButton);
var hdcBtn:HDC;
    hPen,hBrush: HGDIOBJ;
begin
  hdcBtn:=GetDC(Btn.Handle);
  hBrush:= CreateSolidBrush(RGB(0,0,0));
  hPen:=  CreatePen(PS_SOLID,0,RGB(0,0,0));
  SelectObject(hdcBtn,hBrush);
  SelectObject(hdcBtn,hPen);

  Rectangle(hdcBtn, 0, 0, 6, 6);
  Rectangle(hdcBtn, Btn.Width-6, 0, Btn.Width, 6);
  Rectangle(hdcBtn, 0, Btn.Height-6, 6, Btn.Height);
  Rectangle(hdcBtn, Btn.Width-6, Btn.Height-6, Btn.Width, Btn.Height);

  Rectangle(hdcBtn, 0, Btn.Height div 2-3, 6, Btn.Height div 2+3);
  Rectangle(hdcBtn, Btn.Width-6, Btn.Height div 2-3, Btn.Width, Btn.Height div 2+3);
  Rectangle(hdcBtn, Btn.Width div 2-3, 0, Btn.Width div 2+3, 6);
  Rectangle(hdcBtn, Btn.Width div 2-3, Btn.Height-6, Btn.Width div 2+3, Btn.Height);

  DeleteObject(hBrush);
  DeleteObject(hPen);
  ReleaseDC(Btn.Handle,hdcBtn);
end;

procedure TDesignBtn.WMPaint(var Message: TWMPaint);
begin
  inherited;
  if Selected then DrawSquare(Self);
end;

procedure ChangeCursor(Btn:TButton; X,Y:Integer);
begin
  Btn.Cursor:=crDefault; //default state
  if (X<5)Or(Btn.Width-X<5) then Btn.Cursor:=crSizeWE;
  if (Y<5)Or(Btn.Height-Y<5) then Btn.Cursor:=crSizeNS;
  if (X<5)and(Y<5) Or (Btn.Width-X<5)and(Btn.Height-Y<5) then Btn.Cursor:=crSizeNWSE;
  if (Btn.Width-X<5)and(Y<5) Or (X<5)and(Btn.Height-Y<5) then Btn.Cursor:=crSizeNESW;
end;

procedure TDesignBtn.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  Inherited MouseDown(Button,Shift,X,Y);
  if FBtnState=stDesign then begin
     Dragging:=DragBtn; // default state
     if X<5 then Dragging:=ResLeft;
     if Width-X<5 then Dragging:=ResRight;
     if Y<5 then Dragging:=ResTop;
     if Height-Y<5 then Dragging:=ResBottom;
     if (X<5)and(Y<5) then Dragging:=ResLeftTop;
     if (Width-X<5)and(Height-Y<5) then Dragging:=ResRightBottom;
     if (X<5) and(Height-Y<5) then Dragging:=ResLeftBottom;
     if (Width-X<5) and(Y<5) then Dragging:=ResRightTop;
     FixX:=X; FixY:=Y;
     InvalidateRect(Handle,nil,True);
  end;
end;

procedure TDesignBtn.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseMove(Shift, X, Y);
  if FBtnState=stDesign then
    case Dragging of
       ResRight: begin
         Width:=Width+(X-FixX);
         if Width<20 then Width:=20;
         FixX:=X;
       end;
       ResLeft: begin
         Left:=Left+(X-FixX);
         Width:=Width-(X-FixX);
         if Width<20 then Width:=20;
       end;
       ResBottom: begin
         Height:=Height+(Y-FixY);
         if Height<20 then Height:=20;
         FixY:=Y;
       end;
       ResTop: begin
         Top:=Top+(Y-FixY);
         Height:=Height-(Y-FixY);
         if Height<20 then Height:=20;
       end;
       ResLeftTop: begin
         Left:=Left+(X-FixX);
         Width:=Width-(X-FixX);
         if Width<20 then Width:=20;
         Top:=Top+(Y-FixY);
         Height:=Height-(Y-FixY);
         if Height<20 then Height:=20;
       end;
       ResLeftBottom: begin
         Left:=Left+(X-FixX);
         Width:=Width-(X-FixX);
         if Width<20 then Width:=20;
         Height:=Height+(Y-FixY);
         if Height<20 then Height:=20;
         FixY:=Y;
       end;
       ResRightTop: begin
         Width:=Width+(X-FixX);
         if Width<20 then Width:=20;
         FixX:=X;
         Top:=Top+(Y-FixY);
         Height:=Height-(Y-FixY);
         if Height<20 then Height:=20;
       end;
       ResRightBottom: begin
         Width:=Width+(X-FixX);
         if Width<20 then Width:=20;
         FixX:=X;
         Height:=Height+(Y-FixY);
         if Height<20 then Height:=20;
         FixY:=Y;
       end;
       DragBtn:begin
             Left:=Left+X-FixX;
             Top:=Top+Y-FixY;
       end
       else ChangeCursor(Self,X,Y);
    end;
end;

procedure TDesignBtn.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   inherited MouseUp(Button, Shift,X,Y);
   if FBtnState=stDesign then begin
     if Left<0 then Left:=0;
     if Left>Parent.ClientWidth then
                     Left:=Parent.ClientWidth-10;
     if Top<0 then Top:=0;
     if Top>Parent.ClientHeight then
                     Top:=Parent.ClientHeight-10;
     Dragging:=NoDrag;
     InvalidateRect(Handle,nil,True);
   end;
end;

procedure TDesignBtn.DoEnter;
begin
  inherited DoEnter;
  if FBtnState=stDesign then Selected:=True;
end;

procedure TDesignBtn.DoExit;
begin
  inherited DoExit;
  if FBtnState=stDesign then Selected:=False;
end;

procedure Register;
begin
  RegisterComponents('Jeff', [TDesignBtn]);
end;

end.
